#!/bin/bash

set -e

# 
# - for consistency, it is recommended that this script is always
#   run on the same platform (at least where the minor release number
#   remains the same between two releases), otherwise there is a risk
#   that a different version of CMake may produce Makefiles
#   that vary from what has already been tested.
#
# - at the moment, the supported platform for official bootstraps
#
#                Debian 11.x (bullseye)
#
# - to avoid errors about the `zip' and `compress' commands, run
#   the following:
#
#                apt-get install zip ncompress
#
# - note that the configure command below attempts to enable every optional
#   component.  If this is not done, `make dist' will not distribute
#   those components
#

if ! git diff --quiet ;
then
  echo "Uncommitted changes detected, please resolve"
  exit 1
fi

if ! git diff --cached --quiet ;
then
  echo "Staged and uncommitted changes detected, please resolve"
  exit 1
fi

TMP_CLONE=`mktemp -d`
ORIG_DIR=`pwd`
git clone ${ORIG_DIR} ${TMP_CLONE}
cd ${TMP_CLONE}

cmake \
    -DRUN_GPERF=ON \
    -DCMAKE_BUILD_TYPE=Debug \
    -DCMAKE_CXX_FLAGS="${CPPFLAGS} ${CXXFLAGS}" \
    -DCMAKE_C_FLAGS="${CPPFLAGS} ${CFLAGS}" \
    -DWITH_C_ARES=ON \
    -DWITH_SSL=ON \
    -DUSE_POPT=ON \
    -DUSE_SIGCOMP=ON \
    -DUSE_FMT=OFF \
    -DVERSIONED_SONAME=ON \
    -DENABLE_ANDROID=OFF \
    -DUSE_IPV6=ON \
    -DUSE_DTLS=ON \
    -DPEDANTIC_STACK=OFF \
    -DUSE_MYSQL=ON \
    -DUSE_SOCI_POSTGRESQL=ON \
    -DUSE_SOCI_MYSQL=ON \
    -DUSE_POSTGRESQL=ON \
    -DUSE_MAXMIND_GEOIP=ON \
    -DRESIP_HAVE_RADCLI=ON \
    -DUSE_NETSNMP=ON \
    -DBUILD_REPRO=ON \
    -DBUILD_REPRO_DSO_PLUGINS=ON \
    -DBUILD_RETURN=ON \
    -DBUILD_REND=ON \
    -DBUILD_TFM=ON \
    -DBUILD_ICHAT_GW=OFF \
    -DBUILD_TELEPATHY_CM=OFF \
    -DBUILD_RECON=ON \
    -DUSE_SRTP1=OFF \
    -DBUILD_RECONSERVER=ON \
    -DUSE_SIPXTAPI=ON \
    -DUSE_KURENTO=ON \
    -DUSE_GSTREAMER=ON \
    -DUSE_LIBWEBRTC=OFF \
    -DRECON_LOCAL_HW_TESTS=OFF \
    -DDEFAULT_BRIDGE_MAX_IN_OUTPUTS=20 \
    -DBUILD_P2P=OFF \
    -DBUILD_PYTHON=ON \
    -DPYCXX_SRCDIR=/usr/src/CXX \
    -DBUILD_QPID_PROTON=ON \
    -DRESIP_ASSERT_SYSLOG=ON \
    -DREGENERATE_MEDIA_SAMPLES=ON \
    .
#autoreconf --install && \
#  ./configure --with-popt --enable-ipv6 --enable-dtls --with-radcli --with-ssl \
#              --enable-assert-syslog \
#              --enable-android \
#              --with-c-ares \
#              --with-fmt \
#              --with-mysql \
#              --with-postgresql \
#              --with-repro \
#              --with-return \
#              --enable-dso-plugins \
#              --with-python \
#                DEPS_PYTHON_VERSION=`python3 -c "import sys; print('%d.%d' % (sys.version_info[0], sys.version_info[1]))"` \
#                DEPS_PYTHON_CFLAGS="`/usr/bin/python3-config --cflags`" \
#                DEPS_PYTHON_LIBS="`/usr/bin/python3-config --ldflags`" \
#                PYCXX_SRCDIR=/usr/src/CXX/Python3 \
#              --with-rend \
#              --with-tfm \
#              --with-apps \
#              --with-telepathy \
#              --with-ichat-gw \
#              --with-recon \
#              --with-sipxtapi \
#              --with-soci-postgresql \
#              --with-soci-mysql \
#              --with-qpid-proton \
#              --with-geoip \
#              --with-sigcomp \
#              --with-netsnmp \
#              --with-gstreamer \
#              --with-p2p && \
  make dist

cd ${ORIG_DIR}
set -x
cp ${TMP_CLONE}/*.tar.gz .
rm -rf ${TMP_CLONE}

